/*
	Beispiel einer schnellen PCA9554 I2C LCD Interface Funktion

	Die untenstehende Funktion ist ein Beispiel einer schnellen I2C
	LCD Treiber Anwendung. Es hat mich immer gestoert dass bei vielen
	I2C Parallel Expander ICs wie PCF8574, PCA9554 fuer jedes Zeichen drei
	I2C Schreibvorganege notwendig sind. Die untenstehend Function 
	kommt mit nur vier Internen Schreibvorgaengen aus.

	Z.B. Fuer die String "IRREN IST MENSCHLICH" muessen nur 24 Zeichen gesendet 
	werden anstatt der traditionellen 60 bytes. Bei laengeren Texten
	macht sich das angenehm bemerkbar. Das ist eine zweieinhalbfache
	Reduzierung der Uebertragungszeit bei 20 Zeichen.

	Bei 400Khz I2C Taktrate braucht man immerhin nur ca. 400us 
	um dem LCD 14 Zeichen zu senden

	Ich hoffe Ihr findet diese Methode vielleicht nuetzlich.

	Bitte diese Funktion nicht als "Copy and Paste" 
	Bauanleitung betrachten weil es 
	fuer meine Anwendung auf einem eZ ZILOG uC geschrieben ist. 

	Ich wollte den Overhead der function calls vermeiden. Deshalb
	sind keine I2C function calls benutzt worden und alle I2C 
	Aktivititaten direkt mit den uC I2C hardware 
	register verwirklicht.

	Die direkten I2C Register Schreibzugriffe muessen mit den 
	I2C Function Eures Compilers sersetzt werden.

	z.B. 	aus I2CD = c; I2C_WAIT  werden I2C_Write(c);

	LCD Anschluss Details:
	
	PCA9554 Parallel Ausgaenge an LCD Dateneingaenge DB0-DB7
	Drei uC GPIO Ausganege fuer RS, RW, und E

	Wenn das LCD nicht gelesen werden soll, dann muss RW auf Masse gelegt 
	werden.

	Anwendungsbeispiel:
	
	...
	char buffer[32];
	...
	strcpy(buffer, " HALLO FREUNDE ");
	set_LCD_cur(0, 0);
	I2CLCD_prints(buffer);
	...

*/







// Macros used
#define I2C_WAIT       while ((I2CSTAT & TRANSMIT_DATA_REG_EMPTY) == 0x00){;}
#define LOAD_LCD_DATA  PDOUT |= 0x01; asm("NOP"); PDOUT &= 0xFE;


void I2CLCD_prints (char * string)
{
	char c;
	
	PDOUT |= 0x02;	// Set LCD RS to Data Mode

	// Setup PCA9554 for data output 
	I2CCTL |= SEND_START;	// I2C_Start()
   	I2CD = PCA9554_WRITE;   // I2C_write(data)                
   	I2C_WAIT
	I2CD = PCA_OUT_PORT_REG;
   	I2C_WAIT
	
/*
	To make continuous byte transfers possible, first send a data byte to PCA9554
	without loading it into LCD. Then start sending the remaining characters with
	normal LCD load command. This trick avoids the need to send a complete 3-byte
	write sequence to the PCA9554. 
*/
	c = *string++;
	I2CD = c;                            
	I2C_WAIT

	// Begin sequential data transfer to LCD to the end of the string 	
	while (*string)					
	{
		c = *string++;
		I2CD = c;
		I2C_WAIT
		LOAD_LCD_DATA
		_lcd_addr++;
	}
	// Dummy Write to PCA9554 to transfer last data byte into output register
	//c = *string++;
	I2CD = 0x20;                            
	I2C_WAIT
	LOAD_LCD_DATA
	I2C_stop();
}